#include <atlbase.h>
#include <windows.h>
#include <dshow.h>
#include <qedit.h>
#include <string>
#include <vector>
#include <map>

class CameraCapture {
    std::string			_err;

    DWORD			_dwGraphRegister;
    IMediaControl		*_pMC;
    IGraphBuilder		*_pGraph;
    ICaptureGraphBuilder2	*_pCapture;
    ISampleGrabber		*_pGrabber;

    class CGrabCB;
    CGrabCB			*_mygrab;

    int				_width;
    int				_height;

    std::map<std::string, IBaseFilter *> _devices;

    HRESULT GetInterfaces(void);
    HRESULT GetPin(IBaseFilter *, PIN_DIRECTION, IPin **);
    HRESULT ConnectFilters(IGraphBuilder *, IBaseFilter *, IBaseFilter *);
    HRESULT AddGraphToRot(IUnknown *, DWORD *);
    void RemoveGraphFromRot(DWORD);

public:

    CameraCapture();
    ~CameraCapture();

    //
    // Methods that return bool return false on error condition.
    // Error description can be retrieved via GetError().
    //

    // Call FindCaptureDevices(), and then call Init() with one of the returned
    // names.
    //
    // NOTE: You must call this, even if you already know the name of the
    // device.
    bool FindCaptureDevices(std::vector<std::string> &);

    // Call with one of the strings returned from FindCaptureDevices().
    bool Init(const char *capture_device_name);

    // Starts / Stops the capture.
    bool Start();
    bool Stop();

    const char *GetError();

    // Called when video frame is received.
    // The buffer should be 4 * Width() * // Height() bytes long.
    virtual void new_frame(double SampleTime, BYTE *buffer, long len) = 0;

    int Width() { return _width; }
    int Height() { return _height; }
};
